Package: DHARMa
Title: Residual Diagnostics for Hierarchical (Multi-Level / Mixed)
        Regression Models
Version: 0.5.0
Date: 2026-05-06
Authors@R: c(person("Florian", "Hartig", email = "florian.hartig@biologie.uni-regensburg.de", role = c("aut", "cre"), comment=c(ORCID="0000-0002-6255-9059")), person("Lukas", "Lohse", role = "ctb"), person("Melina", "de Souza Leite", role = "ctb", comment=c(ORCID="0000-0003-0505-0667")),  person("Cosmina", "Werneke", role = "ctb", comment=c(ORCID="0009-0008-7629-2173")))
Description: "Uses a simulation-based approach to create readily
  interpretable scaled (quantile) residuals for fitted generalized linear (mixed)
  models. Currently supported are linear and generalized linear (mixed) models from
  'lme4' (classes 'lmerMod', 'glmerMod'), 'glmmTMB', 'GLMMadaptive', 'spaMM', and 'brms'
  (simple models); phylogenetic linear models from 'phylolm' (classes 'phylolm' and 'phyloglm');
  generalized additive models ('gam' from 'mgcv'); 'glm' (including 'negbin' from 'MASS', but excluding
  quasi-distributions) and 'lm' model classes. Moreover, externally created simulations,
  e.g. posterior predictive simulations from Bayesian software such as 'JAGS', 'STAN',
  or 'BUGS' can be processed as well. The resulting residuals are standardized to
  values between 0 and 1 and can be interpreted as intuitively as residuals from a
  linear regression. The package also provides a number of plot and test functions
  for typical model misspecification problems, such as over/underdispersion, zero-inflation,
  and residual spatial, temporal and phylogenetic autocorrelation."
Depends: R (>= 3.5.0)
Imports: stats, graphics, utils, grDevices, Matrix, parallel, gap,
        lmtest, ape, qgam (>= 1.3.2), lme4
Suggests: knitr, testthat (>= 3.0.0), rmarkdown, KernSmooth, sfsmisc,
        MASS, mgcv, mgcViz (>= 0.1.9), spaMM (>= 3.2.0), GLMMadaptive,
        glmmTMB (>= 1.1.2.3), phylolm (>= 2.6.5), brms
Enhances: rstan, phyr, rjags, BayesianTools
License: GPL (>= 3)
URL: http://florianhartig.github.io/DHARMa/
LazyData: TRUE
BugReports: https://github.com/florianhartig/DHARMa/issues
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Encoding: UTF-8
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-26 17:12:33 UTC; florian
Author: Florian Hartig [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6255-9059>),
  Lukas Lohse [ctb],
  Melina de Souza Leite [ctb] (ORCID:
    <https://orcid.org/0000-0003-0505-0667>),
  Cosmina Werneke [ctb] (ORCID: <https://orcid.org/0009-0008-7629-2173>)
Maintainer: Florian Hartig <florian.hartig@biologie.uni-regensburg.de>
Repository: CRAN
Date/Publication: 2026-06-01 07:40:02 UTC
Built: R 4.5.2; ; 2026-06-01 09:46:37 UTC; unix
