# Generated by roxygen2: do not edit by hand

S3method(getData,brmsfit)
S3method(getData,default)
S3method(getData,merMod)
S3method(getFamily,default)
S3method(getFamily,phyloglm)
S3method(getFamily,phylolm)
S3method(getFitted,HLfit)
S3method(getFitted,MixMod)
S3method(getFitted,brmsfit)
S3method(getFitted,default)
S3method(getFitted,gam)
S3method(getFitted,phyloglm)
S3method(getFitted,phylolm)
S3method(getFixedEffects,MixMod)
S3method(getFixedEffects,brmsfit)
S3method(getFixedEffects,default)
S3method(getObservedResponse,HLfit)
S3method(getObservedResponse,default)
S3method(getObservedResponse,phyloglm)
S3method(getObservedResponse,phylolm)
S3method(getPearsonResiduals,brms)
S3method(getPearsonResiduals,default)
S3method(getPearsonResiduals,gam)
S3method(getPredictorNames,MixMod)
S3method(getPredictorNames,default)
S3method(getRefit,HLfit)
S3method(getRefit,MixMod)
S3method(getRefit,brmsfit)
S3method(getRefit,default)
S3method(getRefit,glmmTMB)
S3method(getRefit,lm)
S3method(getRefit,phyloglm)
S3method(getRefit,phylolm)
S3method(getResiduals,MixMod)
S3method(getResiduals,brmsfit)
S3method(getResiduals,default)
S3method(getSimulations,HLfit)
S3method(getSimulations,MixMod)
S3method(getSimulations,brmsfit)
S3method(getSimulations,default)
S3method(getSimulations,gam)
S3method(getSimulations,glmmTMB)
S3method(getSimulations,merMod)
S3method(getSimulations,negbin)
S3method(getSimulations,phyloglm)
S3method(getSimulations,phylolm)
S3method(hist,DHARMa)
S3method(plot,DHARMa)
S3method(plot,DHARMaBenchmark)
S3method(print,DHARMa)
S3method(residuals,DHARMa)
export(benchmarkRuntime)
export(createDHARMa)
export(createData)
export(getData)
export(getFamily)
export(getFitted)
export(getFixedEffects)
export(getObservedResponse)
export(getPearsonResiduals)
export(getPredictorNames)
export(getQuantile)
export(getRandomState)
export(getRefit)
export(getResiduals)
export(getSimulations)
export(outliers)
export(plotConventionalResiduals)
export(plotQQunif)
export(plotResiduals)
export(plotResidualsAll)
export(plotSimulatedResiduals)
export(recalculateResiduals)
export(runBenchmarks)
export(simulateLRT)
export(simulateResiduals)
export(testCategorical)
export(testDispersion)
export(testGeneric)
export(testOutliers)
export(testOverdispersion)
export(testOverdispersionParametric)
export(testPhylogeneticAutocorrelation)
export(testQuantiles)
export(testResiduals)
export(testSimulatedResiduals)
export(testSpatialAutocorrelation)
export(testTemporalAutocorrelation)
export(testUniformity)
export(testZeroInflation)
export(transformQuantiles)
import(grDevices)
import(graphics)
import(stats)
import(utils)
importFrom(lme4,fixef)
importFrom(lme4,refit)
